@@ -14,9 +14,11 @@ use warnings;
 # $| = 1;
 
 my $Time_HiRes_avail;
+my $color_avail;
 BEGIN {
-    $Time_HiRes_avail = 0;
-    $Time_HiRes_avail = eval 'use Time::HiRes qw(time);1';
+    $Time_HiRes_avail = eval 'use Time::HiRes qw(time);1' || 0;
+    $color_avail = eval
+        'use Term::ANSIColor;$Term::ANSIColor::VERSION>=3.00' || 0;
 }
 
 
@@ -35,12 +37,13 @@ BEGIN {
     }
 
     # open a handle to the console, for debugging
-    open *Devel::DumpTrace::TTY, '>>', $^O eq 'MSWin32' ? 'CON' : '/dev/tty';
+    open *Devel::DumpTrace::TTY, '>>',
+        $^O eq 'MSWin32' ? 'CON' : '/dev/tty';
 }
 # compile Devel::DumpTrace::Const AFTER the environment is processed
 use Devel::DumpTrace::Const;
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 our $ARRAY_ELEM_SEPARATOR = ',';
 our $HASH_ENTRY_SEPARATOR = ';';
 our $HASH_PAIR_SEPARATOR = '=>';
@@ -49,6 +52,7 @@ our $SEPARATOR = "-------------------------------------------\n";
 
 my $pid = $$;
 our $DUMPTRACE_FH;
+our $DUMPTRACE_COLOR;
 our $SMART_ABBREV = 1;
 our $DB_ARGS_DEPTH = 3;
 our %EXCLUDE_PKG = ();
@@ -109,20 +113,33 @@ $Devel::DumpTrace::NO_PPI
     } else {
 	$DUMPTRACE_FH = *STDERR;
     }
+    $DUMPTRACE_COLOR = $ENV{DUMPTRACE_COLOR} || '';
+    if ($DUMPTRACE_COLOR) {
+        if ($color_avail) {
+            $DUMPTRACE_COLOR = Term::ANSIColor::color($DUMPTRACE_COLOR);
+            our $DUMPTRACE_RESET = Term::ANSIColor::color('reset');
+        } else {
+            carp "DUMPTRACE_COLOR spec ignored: ",
+                 "Term::ANSIColor not available";
+            $DUMPTRACE_COLOR = '';
+        }
+    } else {
+        $DUMPTRACE_COLOR = "";
+    }
     $SMART_ABBREV = 0 if $ENV{DUMPTRACE_DUMB_ABBREV};
 }
 
 sub import {
     my ($class, @args) = @_;
 
-    push @EXCLUDE_PATTERN, map { '^' . substr($_,1) . '$' } grep { /^-/ } @args;
-    push @INCLUDE_PATTERN, map { '^' . substr($_,1) . '$' } grep { /^\+/ }@args;
-    # these packages will be included/excluded at CHECK time, after 
-    # all packages have been loaded 
+    push @EXCLUDE_PATTERN, map '^' . substr($_,1) . '$', grep { /^-/ } @args;
+    push @INCLUDE_PATTERN, map '^' . substr($_,1) . '$', grep { /^\+/ }@args;
+    # these packages will be included/excluded at CHECK time, after
+    # all packages have been loaded
 
-    push @EXCLUDE_PATTERN, 
+    push @EXCLUDE_PATTERN,
         map { '^' . $_ . '$' } split /,/, $ENV{DUMPTRACE_EXCLPKG} || '';
-    push @INCLUDE_PATTERN, 
+    push @INCLUDE_PATTERN,
         map { '^' . $_ . '$' } split /,/, $ENV{DUMPTRACE_INCLPKG} || '';
 
     @args = grep { /^[^+-]/ } @args;
@@ -172,8 +189,8 @@ sub get_source {
 	    # this happens when we are poking around the symbol table?
 	    # are we corrupting the source file data somehow?
 
-	    $sources{$file} = [ 
-		("SOURCE NOT AVAILABLE FOR file $file: $@") x 999 
+	    $sources{$file} = [
+		("SOURCE NOT AVAILABLE FOR file $file: $@") x 999
 	    ];
 	    if (open my $grrrrr, '<', $file) {
 		$sources{$file} = [ "", <$grrrrr> ];
@@ -321,24 +338,44 @@ sub refresh_pads {
     if ($current >= $target) {
 	save_pads($current - $target);
     }
-    # $current < $target 
+    # $current < $target
     return;
 }
 
+our $last_dumptrace = '';
+our @GT = ("     ",">    ",">>   ",">>>  ",">>>> ",">>>>>");
+
+sub dumptrace {
+    my ($n, $tab, @output) = @_;
+    my $dt = join ('', @output);
+    my $out = join '',($GT[$n],$tab ? "\t         " : " ", $dt);
+    if ($last_dumptrace && $dt eq $last_dumptrace) {
+        # duplicate
+        return;
+    }
+    $last_dumptrace = $dt;
+    if ($DUMPTRACE_COLOR) {
+        our $DUMPTRACE_RESET;
+        $out = join $/, map( $DUMPTRACE_COLOR . $_ . $DUMPTRACE_RESET,
+                               split($/,$out)), "";
+    }
+
+    print {$DUMPTRACE_FH} $out;
+}
+
 sub evaluate_and_display_line {
     my ($code, $p, $f, $l, $sub) = @_;
     my $style = _display_style();
 
     if ($style > DISPLAY_TERSE) {
 	separate();
-	print {$DUMPTRACE_FH} ">>    ",
-	    current_position_string($f,$l,$sub), "\n";
-	print {$DUMPTRACE_FH} ">>> \t\t $code";  # [orig]
+        dumptrace(2,0, current_position_string($f,$l,$sub), "\n");
+        dumptrace(3,1,$code);
     }
 
     # look for assignment operator.
     $DEFERRED{"$sub : $f"} ||= [];
-    if ($code    =~ m{[-+*/&|^.%]?=[^=>]} 
+    if ($code    =~ m{[-+*/&|^.%]?=[^=>]}
 	|| $code =~ m{[\b*&|/<>]{2}=\b}   ) {
 
 	my ($expr1, $op, $expr2) = ($`, $&, $');   # ');
@@ -358,12 +395,12 @@ sub evaluate_and_display_line {
 	  FILE    => $f,
 	  LINE    => $l,
           DISPLAY_FILE_AND_LINE => $style <= DISPLAY_TERSE,
-	  EXPRESSION => [ $expr1, $expr2 ] 
+	  EXPRESSION => [ $expr1, $expr2 ]
 	};
 
 	if ("$expr1$expr2" ne $code) {
 	    if ($style >= DISPLAY_GABBY) {
-		print {$DUMPTRACE_FH} ">>>> \t\t $expr1$expr2";  # [pre eval]
+                dumptrace(4,1,$expr1,$expr2);
 	    }
 	}
 	return;
@@ -392,14 +429,14 @@ sub evaluate_and_display_line {
 	$xcode = perform_variable_substitutions($code, $p);
 
     }
-  
+
     if ($style >= DISPLAY_GABBY) {
 	if ($xcode ne $code) {
-	    print {$DUMPTRACE_FH} ">>>> \t\t $xcode";     # [pre eval]
+	    dumptrace(4,1,$xcode);
 	}
     } elsif ($style == DISPLAY_TERSE) {
-	print {$DUMPTRACE_FH} ">>>>  ", 
-	current_position_string($f,$l,$sub),"\t\t $xcode";
+        dumptrace(4,0, current_position_string($f,$l,$sub),
+                       "\t         $xcode");
     }
     return;
 }
@@ -520,7 +557,7 @@ sub hash_repr {
 	? ref($hashref) . ': ' : '';
     my $maxlen = _abbrev_style() < ABBREV_NONE
 	? _abbrev_style() > ABBREV_STRONG ? 79 : 19 : -1;
-    my $cache_key = join ':', 
+    my $cache_key = join ':',
         $maxlen, $HASH_ENTRY_SEPARATOR, $HASH_PAIR_SEPARATOR;
     my $hash;
 
@@ -681,25 +718,28 @@ sub handle_deferred_output {
 	    $file = $deferred->{FILE};
 	    $line = $deferred->{LINE};
 	}
-	if (defined($line)) {
-	    print {$DUMPTRACE_FH} ">>>>> ", 
-	        current_position_string($file,$line,$deferred->{SUB}), "\t",
-	        perform_extended_variable_substitutions($expr1, $deferred_pkg),
-	        $expr2;
-	} else {
-	    print {$DUMPTRACE_FH} ">>>>>\t\t ",
-	    perform_variable_substitutions($expr1, $deferred_pkg), $expr2;
-	}
+        my $output = $expr2;
+        if (defined($line)) {
+            $output = current_position_string($file,$line,$deferred->{SUB})
+                . "\t" .
+                perform_extended_variable_substitutions($expr1, $deferred_pkg)
+                . $output;
+        } else {
+            $output = "\t         "
+                . perform_variable_substitutions($expr1, $deferred_pkg)
+                . $output;
+        }
+        dumptrace(5,0,$output);
     }
     return;
 }
 
 sub perform_variable_substitutions {
     my ($xcode, $pkg) = @_;
-    $xcode =~ s{  ([\$\@\%])\s*               # sigil
-                  ([\w:]+)                    # package (optional) and var name
-                  (\s*->)?                    # optional dereference op
-                  (\s*[\[\{])?                # optional subscript
+    $xcode =~ s{  ([\$\@\%])\s*           # sigil
+                  ([\w:]+)                # package (optional) and var name
+                  (\s*->)?                # optional dereference op
+                  (\s*[\[\{])?            # optional subscript
                }{ 
 		   evaluate($1,$2,$3||'',$4||'',$pkg) 
                }gex;
@@ -1039,7 +1079,7 @@ Devel::DumpTrace - Evaluate and print out each line before it is executed.
 
 =head1 VERSION
 
-0.19
+0.20
 
 =head1 SYNOPSIS
 
@@ -1204,11 +1244,11 @@ module>.
 
     $ perl -I. -d:DumpTrace=3 littledemo.pl
     >>>>> littledemo.pl:1:[__top__]:  @a:(1,2,3,4,5,6,7,8,9,10,11,12,13,14, \
-        15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37, \
-        38,39,40) = (1 .. 40);
+       15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37, \
+       38,39,40) = (1 .. 40);
     >>>>> littledemo.pl:2:[__top__]:  $b:5 = $a:(1,2,3,4,5,6,7,8,9,10,11,12, \
-        13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35, \
-        36,37,38,39,40)[4];
+       13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35, \
+       36,37,38,39,40)[4];
 
 =item C<$Devel::DumpTrace::TRACE> == 4
 
@@ -1290,9 +1330,9 @@ by setting C<$Devel::DumpTrace::DUMPTRACE_FH> to the desired
 I/O handle:
 
     BEGIN {
-       # if Devel::DumpTrace is loaded, direct output to xtrace-output.txt
+       # if Devel::DumpTrace is loaded, direct output to xtrace.txt
        if ($INC{'Devel/DumpTrace.pm'}) {
-          open $Devel::DumpTrace::DUMPTRACE_FH, '>', '/path/xtrace-output.txt';
+          open $Devel::DumpTrace::DUMPTRACE_FH, '>', '/path/xtrace.txt';
        }
     }
 
@@ -1373,9 +1413,6 @@ The C<DUMPTRACE_NOPPI> variable can be set to force this module
 to use the basic code parser and to ignore the L<PPI|PPI>-based
 version.
 
-B<< This is an incompatible change from v0.06, which recognized 
-the vars C<XTRACE_FH> and C<XTRACE_LEVEL>. >>
-
 If the C<DUMPTRACE_PID> environment variable is set to a true value,
 this module will include process ID information with the file and line
 number in all trace output. This setting can be helpful in debugging
@@ -1387,7 +1424,19 @@ and line number in all trace output. Depending on the evaluation needs
 of each line of the code, the timestamp associated with a line may
 be created either immediately before or immediately after the line
 is executed.
-C<DUMPTRACE_TIME> and C<DUMPTRACE_PID> may both be used at the same time.
+
+If the C<DUMPTRACE_COUNT> environment variable is set to a true value,
+this module will include a count with the file and line number in all
+trace output, indicating how many times your program has visited a
+particular line of code.
+
+The default behaviour of C<Devel::DumpTrace> is to include the name of
+the current subroutine each time the file and line number are displayed.
+If C<DUMPTRACE_NO_SUB> environment variable is set to a true value,
+then the subroutine name will not be displayed.
+
+C<DUMPTRACE_TIME>, C<DUMPTRACE_PID>, C<DUMPTRACE_COUNT>, and
+C<DUMPTRACE_NO_SUB>  may be used separately or in any combination.
 
 When more than one environment variable needs to be set, the caller
 can use the C<DUMPTRACE> environment variable to set multiple variables
@@ -1401,6 +1450,16 @@ is equivalent to the longer
 
     DUMPTRACE_PID=1 DUMPTRACE_FH=trace.out DUMPTRACE_EXCL=My::Module
 
+If C<DUMPTRACE_COLOR> is set, and if the L<Term::ANSIColor|Term::ANSIColor>
+module can be loaded, then C<Devel::DumpTrace> output will be colored in
+the specified color. If your program produces output and you are writing
+C<Devel::DumpTrace> output to your console, the different color of the
+DumpTrace output will help the actual output from the program stand out.
+
+Example:
+
+    DUMPTRACE_COLOR="bold yellow on_black" perl -d:DumpTrace myScript.pl
+
 =cut
 
 Documented in Devel/DumpTrace/PPI.pm:  $ENV{DUMPTRACE_DUMB_ABBREV}
